/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "practico2.h"
#include <QVariant>
#include <QDebug>

Practico2::Practico2(QWidget * parent) : QMainWindow(parent)
{
	// We set the main window
	mainWindow.setupUi(this);

	/*
		Fill the elements combo box. Note that the order must be the same as the
		declared in the header file.
	*/
	mainWindow.elementsComboBox->addItem(tr("Building"));
	mainWindow.elementsComboBox->addItem(tr("Pine"));
	mainWindow.elementsComboBox->addItem(tr("Rocket"));
	mainWindow.elementsComboBox->addItem(tr("Square"));
	mainWindow.elementsComboBox->addItem(tr("Triangle"));

	// Set the counters to -1
	buildingCount = -1;
	pineCount = -1;
	rocketCount = -1;
	squareCount = -1;
	triangleCount = -1;

	/*
		Connect the addPushButton's clicked() signal to our
		addFigureToScene() slot.
	*/
	connect(mainWindow.addPushButton, SIGNAL(clicked()),
	        this, SLOT(addFigureToScene()));

	/*
		Connect the mainWindow's figureListWidget rowChanged() signal to the
		mainWindow's scene figureSelected slot.
		This will trigger an update of the sliders.
	*/
	connect(mainWindow.figureListWidget, SIGNAL(currentRowChanged(int)),
	        mainWindow.scene, SLOT(figureSelected(int)));

	/*
		Connect the signal that sends the sliders parameters to the slot that
		updates them.
	*/
	connect(mainWindow.scene, SIGNAL(changeParameters(double, double,
	                                                  double, double, double)),
	        this, SLOT(resetSliders(double, double, double, double, double)));

	// Connect the sliders to the scene parameters
	connect(mainWindow.xDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeXDisplacement(int)));
	connect(mainWindow.yDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeYDisplacement(int)));
	connect(mainWindow.rotationHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeAngleOfRotation(int)));
	connect(mainWindow.xScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeXScale(int)));
	connect(mainWindow.yScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeYScale(int)));
}

void Practico2::addFigureToScene()
{
	// We must check what kind of figure the user wanted to add
	int element = mainWindow.elementsComboBox->currentIndex();

	if(element == building)
	{
		mainWindow.scene->addBuilding();

		buildingCount++;
		mainWindow.figureListWidget->addItem(tr("Building") +
		                                     QString(" - %1").arg(buildingCount));
	}
	else if(element == pine)
	{
		mainWindow.scene->addPine();

		pineCount++;
		mainWindow.figureListWidget->addItem(tr("Pine") +
		                                     QString(" - %1").arg(pineCount));
	}
	else if(element == rocket)
	{
		mainWindow.scene->addRocket();

		rocketCount++;
		mainWindow.figureListWidget->addItem(tr("Rocket") +
		                                     QString(" - %1").arg(rocketCount));
	}
	else if(element == square)
	{
		mainWindow.scene->addSquare();

		squareCount++;
		mainWindow.figureListWidget->addItem(tr("Square") +
		                                     QString(" - %1").arg(squareCount));
	}
	else if(element == triangle)
	{
		mainWindow.scene->addTriangle();

		triangleCount++;
		mainWindow.figureListWidget->addItem(tr("Triangle") +
		                                     QString(" - %1").arg(triangleCount));
	}
	else
	{
		qDebug() << "element does not match with any figure";
		return ;
	}

	mainWindow.scene->figureSelected(mainWindow.figureListWidget->count() - 1);

	// Now we load the predetermined values
	mainWindow.xDisplacementHorizontalSlider->setValue(0);
	mainWindow.yDisplacementHorizontalSlider->setValue(0);
	mainWindow.rotationHorizontalSlider->setValue(0);
	mainWindow.xScaleHorizontalSlider->setValue(1);
	mainWindow.yScaleHorizontalSlider->setValue(1);
}

void Practico2::resetSliders(double xDisplacement, double yDisplacement,
                             double rotationAngle, double xScale,
                             double yScale)
{
// 	qDebug() << "Sliders reseted" << (int)xDisplacement << xDisplacement;
// 	mainWindow.xDisplacementHorizontalSlider->setValue((int)xDisplacement);
// 	mainWindow.yDisplacementHorizontalSlider->setValue((int)yDisplacement);
// 	mainWindow.rotationHorizontalSlider->setValue((int)rotationAngle);
// 	mainWindow.xScaleHorizontalSlider->setValue((int)xScale);
// 	mainWindow.yScaleHorizontalSlider->setValue((int)yScale);
// 	qDebug() << "Sliders reseted" << (int)xDisplacement << xDisplacement;
}

