/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "scene.h"
#include "matrix3x3.h"
#include "building.h"
#include "pine.h"
#include "rocket.h"
#include "square.h"
#include "triangle.h"
#include <QPainter>
#include <QPaintEvent>
#include <QDebug>
#include <math.h>

#define PI 3.141592654

Scene::Scene(QWidget * parent) : QWidget(parent)
{
	selectedFigure = -1;
}

Scene::~Scene()
{
	// Empty the figures list
	qDeleteAll(figures.begin(), figures.end());
	figures.clear();
}

void Scene::draw(QPainter * painter)
{
	Matrix3X3 * matrix;

	for(int i=0; i< figures.size(); i++)
	{
		matrix = new Matrix3X3();

		matrix->setValue(0, 0, figures[i]->getXScale() *
		                cos(figures[i]->getAngleOfRotation()));
		matrix->setValue(0, 1, figures[i]->getYScale() *
		                -sin(figures[i]->getAngleOfRotation()));
		matrix->setValue(0, 2, figures[i]->getXDisplacement());
		matrix->setValue(1, 0, figures[i]->getXScale() *
		                sin(figures[i]->getAngleOfRotation()));
		matrix->setValue(1, 1, figures[i]->getYScale() *
		                cos(figures[i]->getAngleOfRotation()));
		matrix->setValue(1, 2, figures[i]->getYDisplacement());

		figures[i]->draw(matrix, painter);
		delete matrix;
	}
}

void Scene::addBuilding()
{
	Building * building = new Building();
	figures.append(building);
	update();
}

void Scene::addPine()
{
	Pine * pine = new Pine();
	figures.append(pine);
	update();
}

void Scene::addRocket()
{
	Rocket * rocket = new Rocket();
	figures.append(rocket);
	update();
}

void Scene::addSquare()
{
	Square * square = new Square();
	figures.append(square);
	update();
}

void Scene::addTriangle()
{
	Triangle * triangle = new Triangle();
	figures.append(triangle);
	update();
}

void Scene::figureSelected(int figure)
{
// 	qDebug() << "The selected picture is " << figure;
	if(figure != -1)
	{
// 		qDebug() << "changeParameters emmited" << figure << figures[figure]->getXDisplacement();
		emit changeParameters(figures[figure]->getXDisplacement(),
		                      figures[figure]->getYDisplacement(),
		                      figures[figure]->getAngleOfRotation(),
		                      figures[figure]->getXScale(),
		                      figures[figure]->getYScale());
	}
	selectedFigure = figure;
}

void Scene::changeXDisplacement(int displacement)
{
// 	qDebug() << displacement << selectedFigure;
	if(selectedFigure != -1)
	{
		figures[selectedFigure]->setXDisplacement((double) displacement);
		update();
	}
}

void Scene::changeYDisplacement(int displacement)
{
	if(selectedFigure != -1)
	{
		figures[selectedFigure]->setYDisplacement((double) displacement);
		update();
	}
}

void Scene::changeAngleOfRotation(int angle)
{
	if(selectedFigure != -1)
	{
		// Maths functions are in radians, but sliders are in degrees.
		double radians = (int)angle * PI / 180.0;
		figures[selectedFigure]->setAngleOfRotation(radians);
		update();
	}
}

void Scene::changeXScale(int scale)
{
	if(selectedFigure != -1)
	{
		figures[selectedFigure]->setXScale((double) scale);
		update();
	}
}

void Scene::changeYScale(int scale)
{
	if(selectedFigure != -1)
	{
		figures[selectedFigure]->setYScale((double) scale);
		update();
	}
}

void Scene::paintEvent(QPaintEvent *)
{
	QPainter painter(this);
	painter.setRenderHint(QPainter::Antialiasing);
	painter.setBackground(QBrush(QColor(255, 255, 255)));
	painter.fillRect(0, 0, width(), height(), QBrush(QColor(255, 255, 255)));

	int side = qMin(width(), height());

	painter.setViewport((width()-side)/2, (height()-side)/2, side, side);
	painter.setWindow(-512, -512, 1024, 1024);

	draw(&painter);
}


