/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SCENE
#define SCENE

#include "graphicprimitive.h"
#include <QWidget>

// Forward declarations
class QPainter;
class QPaintEvent;

/// Scene provides the basics for drawing scenes
/**
	TODO: add the documentation here
*/

class Scene : public QWidget
{
	Q_OBJECT

public:
	Scene(QWidget * parent = 0);

	~Scene();

	/// Draw the figures inside the scene
	void draw(QPainter * painter);

	/// Add a building
	void addBuilding();

	/// Add a pine
	void addPine();

	/// Add a rocket
	void addRocket();

	/// Add a square
	void addSquare();

	/// Add a triangle
	void addTriangle();

public slots:
	/// When a figure is selected, do the appropiate
	void figureSelected(int figure);

	/// Change the selected figure x displacement
	void changeXDisplacement(int displacement);

	/// Change the selected figure y displacement
	void changeYDisplacement(int displacement);

	/// Change the selected figure rotation angle
	void changeAngleOfRotation(int angle);

	/// Change the selected figure x scale
	void changeXScale(int scale);

	/// Change the selected figure y scale
	void changeYScale(int scale);

signals:

	/// Send the parameters
	void changeParameters(double xDisplacement, double yDisplacement,
	                      double rotationAngle, double xScale, double yScale);

protected:
	/// Paints the scene in an event
	void paintEvent(QPaintEvent *event);

private:
	QList< GraphicPrimitive * > figures;
	int selectedFigure;
};

#endif // Scene








